/* --------------------------------------------------------

  6.836 RA2 Artificial Chem Simulator 2/20/2002

  FILE: acsim.h

  Author: edsinger@ai.mit, jessical@ai.mit.edu

  This file contains general declarations and defines for the 
  simulation. 

 -------------------------------------------------------------*/
#ifndef ACSIM_H
#define ACSIM_H

#define max(x,y) (x>y?x:y)
#define min(x,y) (x>y?y:x)

#include "math.h"
#include "stdlib.h"
#include "stdio.h"
#include "time.h"
#include "GL/glut.h"

//Platform/Compiler dependent stuff
#ifndef M_PI
#define M_PI 3.1415926535
#endif

#ifndef RAND_MAX 
#define RAND_MAX 2^^15 
#endif 

//------- General World Stuff --------

#define SIZE_WORLD 50	   //size of square torroidal world
#define MAX_BACTERIA 100   //max number of bacteria in the world
#define NEIGHBORHOOD_SIZE 8   //Moore neighborhood
#define MAX_REACTIONS 20   //max number of internal reactions possible

//-------- The Molecules-----------------
/*
  If you add a new molecule, be sure to adjust the structure initializations
  in create_world().
*/
#define NUM_MOLECULES 5			//total number of molecule types in world and bacteria

#define MOL_A 0
#define MOL_B 1
#define MOL_C 2
#define MOL_D 3
#define MOL_E 4

//Molecule labels 
#define MOL_L_SENSE		MOL_A	//left sensor molecule
#define MOL_R_SENSE		MOL_B	//right sensor molecule
#define MOL_R_THRUST	MOL_C	//left thruster molecule
#define MOL_L_THRUST	MOL_D	//right thruster molecule
#define MOL_WORLD		MOL_E


//---------- Useful structures ---------------

struct coord{
   float x;
   float y;
};

struct neighborhood{
   struct coord idx[NEIGHBORHOOD_SIZE];
};

//A discrete patch in the world
struct patch{
   struct coord loc;
   float vat[NUM_MOLECULES]; //world metabolic concentrations 
   struct neighborhood nbh;
};


struct reaction{
   int lhs[NUM_MOLECULES]; //left hand side of a reaction-equation
   int rhs[NUM_MOLECULES]; //right hand side of a reaction-equation
};


struct bacteria{
   float vat[NUM_MOLECULES];				//internal metabolic concentrations 
   struct coord loc;						//grid location
   float direction;						//0-2PI. 0 is E, PI/2 is N
   struct reaction rules[MAX_REACTIONS];	//set of reaction rules
};

//------------ Functions visible to a front end client --------

void	create_world();
void	diffuse_world();
float	get_world_molecular_concentration(int mol, int x, int y);
float	get_world_molecular_scale(int mol);
int	get_bacteria_count();
struct	coord get_bacteria_loc(int id);
float	get_bacteria_dir(int id);
void	bacteria_locomotion(int id);
void	bacteria_sensor(int id);
void	bacteria_reactor(int id);
void	evaporate_world();
#endif
