/* --------------------------------------------------------

   6.836 RA2 Artificial Chem Simulator 2/20/2002
   
   Author: edsinger@ai.mit, jessical@ai.mit.edu
   
   This file contains the code for the visualization of the simulator
   It uses the glut library. 
 -------------------------------------------------------------*/
#include "acsim.h"

#define IMG_SCALE 5 		 //scale the world size
#define IMG_MOLECULE MOL_WORLD	 //which molecular concentration to view


void 
renderScene(void) {
   
   float patch_size,xcoord,ycoord,val,dir,scale;
   int x,y,nBact,b;
   struct coord loc,cb,c1,c2,c3;
   
   glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
   glClear( GL_COLOR_BUFFER_BIT );
   glPushMatrix();
   
   //-----------------  Draw each patch in the World  ---------------------
   patch_size = (float)IMG_SCALE/(float)SIZE_WORLD ;
   scale = get_world_molecular_scale(IMG_MOLECULE);
   for (x =0;x<SIZE_WORLD;x++)
      for (y =0;y<SIZE_WORLD;y++) {
	 xcoord = (float)((2.0*(float)x / (float)SIZE_WORLD)-1.0);
	 ycoord = (float)((2.0*(float)y / (float)SIZE_WORLD)-1.0);
	 
	 val = get_world_molecular_concentration(IMG_MOLECULE, x, y);
	 val = val * scale;

	 // Set the patch color
	 glColor3f(val,val,0.0f);

	 // Start Drawing Quads
	 glBegin(GL_QUADS);

	 // Left And Up 1 Unit (Top Left)
	 glVertex3f(xcoord, ycoord, 0.0f);

	 // Right And Up 1 Unit (Top Right)
	 glVertex3f( xcoord+patch_size, ycoord, 0.0f);

	 // Right And Down One Unit (Bottom Right)
	 glVertex3f( xcoord+patch_size,ycoord+patch_size, 0.0f);

	 // Left And Down One Unit (Bottom Left)
	 glVertex3f(xcoord,ycoord+patch_size, 0.0f);

	 glEnd();
      }
	 
	 //---------------------  Draw the Bacteria -------------------------------
   
	 nBact = get_bacteria_count();
	 for (b=0;b<nBact;b++) {
	    loc = get_bacteria_loc(b);
	    dir = get_bacteria_dir(b);
	    cb.x = (float)((2.0*loc.x / (float)SIZE_WORLD)-1.0);
	    cb.y = (float)((2.0*loc.y / (float)SIZE_WORLD)-1.0);
	    //Form the triangle
	    c1.x = cb.x + patch_size* cos(dir);
	    c1.y = cb.y + patch_size* sin(dir);
	    c2.x = cb.x + patch_size* cos(dir + M_PI/2)/4;
	    c2.y = cb.y + patch_size* sin(dir + M_PI/2)/4;
	    c3.x = cb.x + patch_size* cos(dir - M_PI/2)/4;
	    c3.y = cb.y + patch_size* sin(dir - M_PI/2)/4;
	    // Set the bacteria color
	    glColor3f(0,0,1.0f);
	    glBegin(GL_TRIANGLES);	
	    glVertex3f(c1.x, c1.y, 0.0f);
	    glVertex3f(c2.x, c2.y, 0.0f);
	    glVertex3f(c3.x, c3.y, 0.0f);
	    glEnd();
	 }
   
	 glPopMatrix();
	 glutSwapBuffers();
   
	 //------------------ Advance world one time step ----------------------
	 diffuse_world();
	 //evaporate_world();  //you can try this out for fun
	 
	 for (b=0;b<nBact;b++) {
	    bacteria_sensor(b);
	    bacteria_reactor(b);
	    bacteria_locomotion(b);
	 } 
}
   
   
int 
main(int argc, char *argv[]) {
   glutInit(&argc, argv);
   
   // This is where we say that we want a double buffer
   glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
   glutInitWindowPosition(100,100);
   glutInitWindowSize( SIZE_WORLD*IMG_SCALE, SIZE_WORLD*IMG_SCALE );
   glutCreateWindow("6.826 Artificial Chem Sim");
   glutDisplayFunc(renderScene);
   glutIdleFunc(renderScene);
   
   //---------------------- Setup Simulation -----------------
   create_world();
   
   //---------------------------------------------------------   
   glutMainLoop();
}


